@@ -24,19 +24,33 @@ from aiohomekit.crypto.chacha20poly1305 import (
 )
 from aiohomekit.exceptions import (
     AccessoryDisconnectedError,
+    AccessoryNotFoundError,
     AuthenticationError,
     ConnectionError,
     HomeKitException,
+    HttpErrorResponse,
     TimeoutError,
 )
 from aiohomekit.http import HttpContentTypes
 from aiohomekit.http.response import HttpResponse
 from aiohomekit.protocol import get_session_keys
 from aiohomekit.protocol.tlv import TLV
+from aiohomekit.zeroconf import async_find_device_ip_and_port
 
 logger = logging.getLogger(__name__)
 
 
+def serialize_json(obj) -> bytes:
+    """
+    An iPhone sends JSON like this:
+
+    {"characteristics":[{"iid":15,"aid":2,"ev":true}]}
+
+    Some devices (Tado Internet Bridge) depend on this some of the time.
+    """
+    return json.dumps(obj, separators=(",", ":")).encode("utf-8")
+
+
 class InsecureHomeKitProtocol(asyncio.Protocol):
     def __init__(self, connection):
         self.connection = connection
@@ -70,7 +84,7 @@ class InsecureHomeKitProtocol(asyncio.Protocol):
         self.result_cbs.append(result)
 
         try:
-            return await asyncio.wait_for(result, 10)
+            return await asyncio.wait_for(result, 30)
         except asyncio.TimeoutError:
             self.transport.write_eof()
             self.transport.close()
@@ -183,7 +197,7 @@ class SecureHomeKitProtocol(InsecureHomeKitProtocol):
 
 
 class HomeKitConnection:
-    def __init__(self, owner, host, port):
+    def __init__(self, owner, host, port, concurrency_limit=1):
         self.owner = owner
         self.host = host
         self.port = port
@@ -199,6 +213,8 @@ class HomeKitConnection:
 
         self.is_secure = False
 
+        self._concurrency_limit = asyncio.Semaphore(concurrency_limit)
+
     @property
     def is_connected(self):
         return self.transport and self.protocol and not self.closed
@@ -259,7 +275,7 @@ class HomeKitConnection:
 
     async def put_json(self, target, body):
         response = await self.put(
-            target, json.dumps(body).encode("utf-8"), content_type=HttpContentTypes.TLV,
+            target, serialize_json(body), content_type=HttpContentTypes.JSON,
         )
 
         if response.code == 204:
@@ -297,7 +313,7 @@ class HomeKitConnection:
 
     async def post_json(self, target, body):
         response = await self.post(
-            target, json.dumps(body).encode("utf-8"), content_type=HttpContentTypes.TLV,
+            target, serialize_json(body), content_type=HttpContentTypes.JSON,
         )
 
         if response.code != 204:
@@ -321,9 +337,13 @@ class HomeKitConnection:
         return parsed
 
     async def post_tlv(self, target, body, expected=None):
-        response = await self.post(
-            target, TLV.encode_list(body), content_type=HttpContentTypes.TLV,
-        )
+        try:
+            response = await self.post(
+                target, TLV.encode_list(body), content_type=HttpContentTypes.TLV,
+            )
+        except HttpErrorResponse as e:
+            self.transport.close()
+            response = e.response
         body = TLV.decode_bytes(response.body, expected=expected)
         return body
 
@@ -339,6 +359,11 @@ class HomeKitConnection:
         :param headers: a list of (header, value) tuples (optional)
         :param body: The body of the request (optional)
         """
+        if not self.protocol:
+            raise AccessoryDisconnectedError(
+                "Connection lost before request could be sent"
+            )
+
         buffer = []
         buffer.append(
             "{method} {target} HTTP/1.1".format(method=method.upper(), target=target,)
@@ -368,7 +393,20 @@ class HomeKitConnection:
         # https://github.com/jlusiardi/homekit_python/issues/12
         # https://github.com/jlusiardi/homekit_python/issues/16
 
-        return await self.protocol.send_bytes(request_bytes)
+        async with self._concurrency_limit:
+            logger.debug("%s: raw request: %r", self.host, request_bytes)
+            resp = await self.protocol.send_bytes(request_bytes)
+
+        if resp.code >= 400 and resp.code <= 499:
+            logger.debug(f"Got HTTP error {resp.code} for {method} against {target}")
+            raise HttpErrorResponse(
+                f"Got HTTP error {resp.code} for {method} against {target}",
+                response=resp,
+            )
+
+        logger.debug("%s: raw response: %r", self.host, resp.body)
+
+        return resp
 
     async def close(self):
         """
@@ -488,6 +526,13 @@ class SecureHomeKitConnection(HomeKitConnection):
     async def _connect_once(self):
         self.is_secure = False
 
+        try:
+            self.host, self.port = await async_find_device_ip_and_port(
+                self.pairing_data["AccessoryPairingID"]
+            )
+        except AccessoryNotFoundError:
+            pass
+
         await super()._connect_once()
 
         state_machine = get_session_keys(self.pairing_data)
