@@ -17,8 +17,20 @@
 from .characteristic import Characteristic
 from .characteristic_formats import CharacteristicFormats
 from .characteristic_types import CharacteristicsTypes
-from .const import CurrentMediaStateValues, RemoteKeyValues, TargetMediaStateValues
+from .const import (
+    CurrentMediaStateValues,
+    HeatingCoolingCurrentValues,
+    HeatingCoolingTargetValues,
+    InputEventValues,
+    InUseValues,
+    IsConfiguredValues,
+    ProgramModeValues,
+    RemoteKeyValues,
+    TargetMediaStateValues,
+    ValveTypeValues,
+)
 from .permissions import CharacteristicPermissions
+from .types import CharacteristicShortUUID, CharacteristicUUID
 from .units import CharacteristicUnits
 
 __all__ = [
@@ -30,4 +42,13 @@ __all__ = [
     "TargetMediaStateValues",
     "CurrentMediaStateValues",
     "RemoteKeyValues",
+    "InputEventValues",
+    "HeatingCoolingCurrentValues",
+    "HeatingCoolingTargetValues",
+    "CharacteristicUUID",
+    "CharacteristicShortUUID",
+    "InUseValues",
+    "IsConfiguredValues",
+    "ProgramModeValues",
+    "ValveTypeValues",
 ]
